
package org.eclipse.birt.report.engine.internal.executor.doc;

import java.io.File;
import java.util.ArrayList;

import org.eclipse.birt.report.engine.EngineCase;
import org.eclipse.birt.report.engine.api.EngineConfig;
import org.eclipse.birt.report.engine.api.IReportDocument;
import org.eclipse.birt.report.engine.api.IReportEngine;
import org.eclipse.birt.report.engine.api.IReportRunnable;
import org.eclipse.birt.report.engine.api.IRunTask;
import org.eclipse.birt.report.engine.api.impl.ReportEngine;
import org.eclipse.birt.report.engine.content.IContent;
import org.eclipse.birt.report.engine.content.ILabelContent;
import org.eclipse.birt.report.engine.content.IPageContent;
import org.eclipse.birt.report.engine.content.IReportContent;
import org.eclipse.birt.report.engine.content.ITableBandContent;
import org.eclipse.birt.report.engine.content.ITableContent;
import org.eclipse.birt.report.engine.executor.ExecutionContext;
import org.eclipse.birt.report.engine.executor.IReportExecutor;
import org.eclipse.birt.report.engine.executor.IReportItemExecutor;
import org.eclipse.birt.report.engine.internal.document.IReportContentLoader;

public class ReportReaderTest extends EngineCase
{

	static final String UTEST_FOLDER = "./utest/ReportReaderTest/";
	static final String REPORT_DESIGN_RESOURCE = "org/eclipse/birt/report/engine/internal/executor/doc/mutiple-page-report.rptdesign";
	static final String REPORT_DESIGN_FILE = UTEST_FOLDER
			+ "mutiple-page-report.rptdesign";
	static final String REPORT_DOCUMENT = UTEST_FOLDER
			+ "mutiple-page-report.rptdocument";

	IReportEngine engine;

	public void setUp( )
	{
		removeFile( UTEST_FOLDER );
		new File( UTEST_FOLDER ).mkdirs( );
		copyResource( REPORT_DESIGN_RESOURCE, REPORT_DESIGN_FILE );
		engine = new ReportEngine( new EngineConfig( ) );
	}

	public void tearDown( )
	{
		removeFile( REPORT_DESIGN_FILE );
		removeFile( REPORT_DOCUMENT );
		removeFile( UTEST_FOLDER );
	}

	public void testReader( ) throws Exception
	{
		IReportRunnable runnable = engine.openReportDesign( REPORT_DESIGN_FILE );
		IRunTask runTask = engine.createRunTask( runnable );
		runTask.run( REPORT_DOCUMENT );
		runTask.close( );

		doTestReport( );
		doTestReportlet( );
		doTestPage( );
		doTestPageWithPaginate( );
		doTestPageWithTable( );
	}

	public void doTestReport( ) throws Exception
	{
		IReportDocument document = engine.openReportDocument( REPORT_DOCUMENT );
		ExecutionContext context = new ExecutionContext( );
		context.setReportDocument( document );
		IReportRunnable runnable = document.getReportRunnable( );
		context.setRunnable( runnable );

		IReportExecutor reader = new ReportReader( context );

		IReportContent report = reader.execute( );
		assertTrue( report != null );

		IReportItemExecutor childExecutor = reader.getNextChild( );
		IContent child = childExecutor.execute( );
		childExecutor.close( );
		assertTrue( child instanceof ILabelContent );

		childExecutor = reader.getNextChild( );
		child = childExecutor.execute( );
		childExecutor.close( );
		assertTrue( child instanceof ILabelContent );

		childExecutor = reader.getNextChild( );
		child = childExecutor.execute( );
		childExecutor.close( );
		assertTrue( child instanceof ILabelContent );

		childExecutor = reader.getNextChild( );
		child = childExecutor.execute( );
		childExecutor.close( );
		assertTrue( child instanceof ITableContent );

		assertTrue( !reader.hasNextChild( ) );
		reader.close( );

		document.close( );
	}

	protected void doTestReportlet( ) throws Exception
	{
		IReportDocument document = engine.openReportDocument( REPORT_DOCUMENT );
		ExecutionContext context = new ExecutionContext( );
		context.setReportDocument( document );
		IReportRunnable runnable = document.getReportRunnable( );
		context.setRunnable( runnable );

		long reportlet = document.getBookmarkOffset( "reportlet" );
		assertTrue( reportlet != -1 );

		IReportExecutor reader = new ReportletReader( context, reportlet );

		IReportContent report = reader.execute( );
		assertTrue( report != null );

		IReportItemExecutor childExecutor = reader.getNextChild( );
		IContent child = childExecutor.execute( );
		childExecutor.close( );
		assertTrue( child instanceof ILabelContent );

		assertTrue( !reader.hasNextChild( ) );
		reader.close( );

		document.close( );

	}

	protected void doTestPage( ) throws Exception
	{
		IReportDocument document = engine.openReportDocument( REPORT_DOCUMENT );
		ExecutionContext context = new ExecutionContext( );
		context.setReportDocument( document );
		IReportRunnable runnable = document.getReportRunnable( );
		context.setRunnable( runnable );

		IReportExecutor reader = new ReportPageReader( context, 2,
				IReportContentLoader.MULTI_PAGE );

		IReportContent report = reader.execute( );
		assertTrue( report != null );

		IReportItemExecutor pageExecutor = reader.getNextChild( );
		IPageContent page = (IPageContent)pageExecutor.execute( );
		assertEquals( 1, page.getPageHeader( ).getChildren( ).size( ) );
		assertEquals( 1, page.getPageFooter( ).getChildren( ).size( ) );

		IReportItemExecutor childExecutor = pageExecutor.getNextChild( );
		IContent child = childExecutor.execute( );
		childExecutor.close( );
		assertTrue( child instanceof ILabelContent );

		assertTrue( !pageExecutor.hasNextChild( ) );
		pageExecutor.close( );

		assertTrue( !reader.hasNextChild( ) );
		reader.close( );

		document.close( );
	}

	protected void doTestPageWithPaginate( ) throws Exception
	{
		IReportDocument document = engine.openReportDocument( REPORT_DOCUMENT );
		ExecutionContext context = new ExecutionContext( );
		context.setReportDocument( document );
		IReportRunnable runnable = document.getReportRunnable( );
		context.setRunnable( runnable );

		ArrayList pageLists = new ArrayList( );
		pageLists.add( new long[]{2, 3} );
		IReportExecutor reader = new ReportPageReader( context, pageLists,
				IReportContentLoader.NO_PAGE );

		IReportContent report = reader.execute( );
		assertTrue( report != null );

		IReportItemExecutor childExecutor = reader.getNextChild( );
		IContent child = childExecutor.execute( );
		childExecutor.close( );
		assertTrue( child instanceof ILabelContent );

		childExecutor = reader.getNextChild( );
		child = childExecutor.execute( );
		childExecutor.close( );
		assertTrue( child instanceof ILabelContent );

		assertTrue( !reader.hasNextChild( ) );
		reader.close( );

		document.close( );
	}
	protected void doTestPageWithTable( ) throws Exception
	{
		IReportDocument document = engine.openReportDocument( REPORT_DOCUMENT );
		ExecutionContext context = new ExecutionContext( );
		context.setReportDocument( document );
		IReportRunnable runnable = document.getReportRunnable( );
		context.setRunnable( runnable );

		IReportExecutor reader = new ReportPageReader( context, 4,
				IReportContentLoader.MULTI_PAGE );

		IReportContent report = reader.execute( );
		assertTrue( report != null );

		IReportItemExecutor pageExecutor = reader.getNextChild( );
		IPageContent page = (IPageContent)pageExecutor.execute( );
		assertEquals( 1, page.getPageHeader( ).getChildren( ).size( ) );
		assertEquals( 1, page.getPageFooter( ).getChildren( ).size( ) );

		IReportItemExecutor tableExecutor = pageExecutor.getNextChild( );
		IContent table = tableExecutor.execute( );
		
		IReportItemExecutor childExecutor = tableExecutor.getNextChild( );
		IContent child = childExecutor.execute( );
		assertTrue( child instanceof ITableBandContent );
		childExecutor.close( );
		
		tableExecutor.close( );
		assertTrue( table instanceof ITableContent );
		
		assertTrue( !pageExecutor.hasNextChild( ) );
		pageExecutor.close( );

		assertTrue( !reader.hasNextChild( ) );
		reader.close( );

		document.close( );
	}
	
}
