/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.model.api;

import java.util.Iterator;

import org.eclipse.birt.report.model.api.elements.DesignChoiceConstants;
import org.eclipse.birt.report.model.api.elements.structures.Action;
import org.eclipse.birt.report.model.api.elements.structures.DataSetParameter;
import org.eclipse.birt.report.model.api.elements.structures.OdaDataSetParameter;
import org.eclipse.birt.report.model.api.elements.structures.ParamBinding;
import org.eclipse.birt.report.model.elements.OdaDataSource;
import org.eclipse.birt.report.model.util.BaseTestCase;

/**
 * Test cases for DataSetHandle class.
 * <p>
 * <table border="1" cellpadding="2" cellspacing="2" style="border-collapse:
 * collapse" bordercolor="#111111">
 * <th width="20%">Method</th>
 * <th width="40%">Test Case</th>
 * <th width="40%">Expected</th>
 * <tr>
 * <td>{@link #testProperties()}</td>
 * <td>Reads and Sets properties of a dataset.</td>
 * <td>Values are returned and written correctly.</td>
 * </tr>
 * </table>
 */

public class DataSetHandleTest extends BaseTestCase
{

	String fileName = "DataSetHandleTest.xml"; //$NON-NLS-1$
	String outFileName = "DataSetHandleTest_out.xml"; //$NON-NLS-1$

	/*
	 * @see BaseTestCase#setUp()
	 */

	protected void setUp( ) throws Exception
	{
		super.setUp( );
	}

	/**
	 * Tests all properties on an data set. JdbcSelectDataSet is used.
	 * 
	 * @throws Exception
	 */

	public void testProperties( ) throws Exception
	{
		OdaDataSetHandle dataSet = getDataSet( );

		OdaDataSetHandle emptyDsHandle = (OdaDataSetHandle) designHandle
				.findDataSet( "Data Set" ); //$NON-NLS-1$

		emptyDsHandle.setQueryText( " " ); //$NON-NLS-1$
		assertEquals( " ", emptyDsHandle.getQueryText( ) ); //$NON-NLS-1$

		emptyDsHandle.setQueryText( "" ); //$NON-NLS-1$
		assertEquals( "", emptyDsHandle.getQueryText( ) ); //$NON-NLS-1$

		// Test DataSet property

		assertEquals( "myDataSource", dataSet.getDataSource( ).getName( ) ); //$NON-NLS-1$
		assertEquals( "myDataSource", dataSet.getDataSourceName( ) ); //$NON-NLS-1$
		assertEquals( "script_beforeopen", dataSet.getBeforeOpen( ) ); //$NON-NLS-1$
		assertEquals( "script_beforeclose", dataSet.getBeforeClose( ) ); //$NON-NLS-1$
		assertEquals( "script_onfetch", dataSet.getOnFetch( ) ); //$NON-NLS-1$
		assertEquals( "script_afteropen", dataSet.getAfterOpen( ) ); //$NON-NLS-1$
		assertEquals( "script_afterclose", dataSet.getAfterClose( ) ); //$NON-NLS-1$

		// Test "input-parameters" on DataSet

		Iterator parameters = dataSet.parametersIterator( );

		DataSetParameterHandle parameter = (DataSetParameterHandle) parameters
				.next( );

		assertEquals( 1, parameter.getPosition( ).intValue( ) );
		assertEquals( "name", parameter.getName( ) ); //$NON-NLS-1$
		assertEquals( DesignChoiceConstants.COLUMN_DATA_TYPE_STRING, parameter
				.getDataType( ) );
		assertTrue( parameter.isOptional( ) );
		assertEquals( "default value 1", parameter.getDefaultValue( ) ); //$NON-NLS-1$
		assertTrue( parameter.isInput( ) );
		assertFalse( parameter.isOutput( ) );

		parameter.setDefaultValue( "new default value 1" ); //$NON-NLS-1$
		assertEquals( "new default value 1", parameter.getDefaultValue( ) ); //$NON-NLS-1$

		// 3 parameters totally.

		assertTrue( parameters.hasNext( ) );
		parameters.next( );
		assertTrue( parameters.hasNext( ) );
		parameters.next( );
		assertTrue( parameters.hasNext( ) );

		// Test "output-parameters" on DataSet

		parameter = (DataSetParameterHandle) parameters.next( );
		assertEquals( "birth", parameter.getName( ) ); //$NON-NLS-1$
		assertEquals( DesignChoiceConstants.COLUMN_DATA_TYPE_DATETIME,
				parameter.getDataType( ) );
		assertTrue( parameter.isOutput( ) );

		parameter = (DataSetParameterHandle) parameters.next( );
		assertEquals( "title", parameter.getName( ) ); //$NON-NLS-1$
		assertEquals( DesignChoiceConstants.COLUMN_DATA_TYPE_STRING, parameter
				.getDataType( ) );

		assertFalse( parameters.hasNext( ) );

		// Test "param-bindings" on DataSet

		Iterator bindings = dataSet.paramBindingsIterator( );

		ParamBindingHandle binding = (ParamBindingHandle) bindings.next( );
		assertEquals( "param1", binding.getParamName( ) ); //$NON-NLS-1$
		assertEquals( "value1", binding.getExpression( ) ); //$NON-NLS-1$

		assertNotNull( bindings.next( ) );
		assertNull( bindings.next( ) );

		// Test "result-set" on DataSet

		Iterator columns = dataSet.resultSetIterator( );

		ResultSetColumnHandle column = (ResultSetColumnHandle) columns.next( );
		assertEquals( 1, column.getPosition( ).intValue( ) );
		assertEquals( "name", column.getColumnName( ) ); //$NON-NLS-1$
		assertEquals( DesignChoiceConstants.COLUMN_DATA_TYPE_STRING, column
				.getDataType( ) );

		assertNotNull( columns.next( ) );
		assertNotNull( columns.next( ) );
		assertNull( columns.next( ) );

		// Test "computed-columns" on DataSet

		columns = dataSet.computedColumnsIterator( );

		ComputedColumnHandle computedColumn = (ComputedColumnHandle) columns
				.next( );
		assertEquals( "column1", computedColumn.getName( ) ); //$NON-NLS-1$
		assertEquals( "expression1", computedColumn.getExpression( ) ); //$NON-NLS-1$
		assertEquals( DesignChoiceConstants.COLUMN_DATA_TYPE_INTEGER,
				computedColumn.getDataType( ) );
		computedColumn = (ComputedColumnHandle) columns.next( );
		assertEquals( "column2", computedColumn.getName( ) ); //$NON-NLS-1$
		assertEquals( "expression2", computedColumn.getExpression( ) ); //$NON-NLS-1$
		assertEquals( DesignChoiceConstants.COLUMN_DATA_TYPE_DATETIME,
				computedColumn.getDataType( ) );
		computedColumn = (ComputedColumnHandle) columns.next( );
		assertEquals( "column3", computedColumn.getName( ) ); //$NON-NLS-1$
		assertEquals( "expression3", computedColumn.getExpression( ) ); //$NON-NLS-1$
		assertEquals( DesignChoiceConstants.COLUMN_DATA_TYPE_ANY,
				computedColumn.getDataType( ) );

		// Test "column-hints" on DataSet

		Iterator columnHints = dataSet.columnHintsIterator( );

		ColumnHintHandle columnHint = (ColumnHintHandle) columnHints.next( );

		assertEquals( "username", columnHint.getColumnName( ) ); //$NON-NLS-1$
		assertEquals( "userid", columnHint.getAlias( ) ); //$NON-NLS-1$
		assertEquals( DesignChoiceConstants.SEARCH_TYPE_NONE, columnHint
				.getSearching( ) );
		assertEquals( DesignChoiceConstants.EXPORT_TYPE_ALWAYS, columnHint
				.getExport( ) );
		assertEquals( DesignChoiceConstants.ANALYSIS_TYPE_DIMENSION, columnHint
				.getAnalysis( ) );
		assertEquals( "4", columnHint.getParentLevel( ) ); //$NON-NLS-1$
		assertEquals( "##.###", columnHint.getFormat( ) ); //$NON-NLS-1$
		assertEquals( "message.column-hint.username", columnHint //$NON-NLS-1$
				.getDisplayNameKey( ) );
		assertEquals( "User Name", columnHint.getDisplayName( ) ); //$NON-NLS-1$
		assertEquals( "message.column-hint.help", columnHint //$NON-NLS-1$
				.getHelpTextKey( ) );
		assertEquals( "Help me!", columnHint.getHelpText( ) ); //$NON-NLS-1$

		// Test "filter" on DataSet

		Iterator filters = dataSet.filtersIterator( );

		FilterConditionHandle filterHandle = (FilterConditionHandle) filters
				.next( );

		assertEquals( "lt", filterHandle.getOperator( ) ); //$NON-NLS-1$
		assertEquals( "filter expression", filterHandle.getExpr( ) ); //$NON-NLS-1$
		assertEquals( "value1 expression", filterHandle.getValue1( ) ); //$NON-NLS-1$
		assertEquals( "value2 expression", filterHandle.getValue2( ) ); //$NON-NLS-1$

		assertNull( filters.next( ) );

		// writes properties to get a new file.

		OdaDataSource dataSource2 = (OdaDataSource) design
				.findDataSource( "myDataSource2" ); //$NON-NLS-1$

		// Change DataSet property

		dataSet.setDataSource( dataSource2.getName( ) );
		dataSet.setBeforeOpen( "New before open" ); //$NON-NLS-1$
		dataSet.setBeforeClose( "New before close" ); //$NON-NLS-1$
		dataSet.setAfterOpen( "New after open" ); //$NON-NLS-1$
		dataSet.setAfterClose( "New after open" ); //$NON-NLS-1$
		dataSet.setOnFetch( "New on fetch" ); //$NON-NLS-1$

		// Change "input-parameters" on DataSet

		parameters = dataSet.parametersIterator( );
		parameter = (DataSetParameterHandle) parameters.next( );

		parameter = (DataSetParameterHandle) parameters.next( );
		parameter.setPosition( Integer.valueOf( "91" ) ); //$NON-NLS-1$
		parameter.setName( "new name" ); //$NON-NLS-1$
		parameter.setDataType( DesignChoiceConstants.COLUMN_DATA_TYPE_FLOAT );
		parameter.setIsOptional( false );

		// Change "output-parameters" on DataSet

		parameter = (DataSetParameterHandle) parameters.next( );
		parameter.setName( "new name" ); //$NON-NLS-1$
		parameter.setDataType( DesignChoiceConstants.COLUMN_DATA_TYPE_FLOAT );
		parameter.setPosition( Integer.valueOf( "91" ) ); //$NON-NLS-1$

		// Change "param-binding" on DataSet

		bindings = dataSet.paramBindingsIterator( );

		binding = (ParamBindingHandle) bindings.next( );
		binding.setParamName( "new param1" ); //$NON-NLS-1$
		binding.setExpression( "new value1" ); //$NON-NLS-1$

		binding = (ParamBindingHandle) bindings.next( );

		binding.setParamName( "new param2" ); //$NON-NLS-1$
		binding.setParamName( "new value2" ); //$NON-NLS-1$

		// Change "result-set" on DataSet

		columns = dataSet.resultSetIterator( );

		column = (ResultSetColumnHandle) columns.next( );

		column.setPosition( Integer.valueOf( "81" ) ); //$NON-NLS-1$
		column.setColumnName( "new name" ); //$NON-NLS-1$
		column.setDataType( DesignChoiceConstants.COLUMN_DATA_TYPE_FLOAT );

		// Change "computed-columns" in DataSet

		columns = dataSet.computedColumnsIterator( );
		computedColumn = (ComputedColumnHandle) columns.next( );

		computedColumn.setName( "new column1" ); //$NON-NLS-1$
		computedColumn.setExpression( "new expression 1" ); //$NON-NLS-1$
		computedColumn
				.setDataType( DesignChoiceConstants.COLUMN_DATA_TYPE_DECIMAL );

		computedColumn = (ComputedColumnHandle) columns.next( );
		computedColumn.setName( "new column2" ); //$NON-NLS-1$
		computedColumn.setExpression( "new expression 2" ); //$NON-NLS-1$

		// Change "column-hints" on DataSet

		columnHints = dataSet.columnHintsIterator( );

		columnHint = (ColumnHintHandle) columnHints.next( );

		columnHint.setColumnName( "new username" ); //$NON-NLS-1$
		columnHint.setAlias( "new userid" ); //$NON-NLS-1$
		columnHint.setSearching( DesignChoiceConstants.SEARCH_TYPE_INDEXED );
		columnHint.setExport( DesignChoiceConstants.EXPORT_TYPE_IF_REALIZED );
		columnHint.setAnalysis( DesignChoiceConstants.ANALYSIS_TYPE_DETAIL );
		columnHint.setParentLevel( "new level" ); //$NON-NLS-1$
		columnHint.setFormat( "new format" ); //$NON-NLS-1$
		columnHint.setDisplayNameKey( "new display name id" ); //$NON-NLS-1$
		columnHint.setDisplayName( "new display name" ); //$NON-NLS-1$
		columnHint.setHelpTextKey( "new help text id" ); //$NON-NLS-1$
		columnHint.setHelpText( "new help text" ); //$NON-NLS-1$	
	}

	public void testDataSetParameterSynchronization( ) throws Exception
	{
		designHandle = createDesign( );

		// Add one data set to design

		DataSetHandle dataSetHandle = designHandle.getElementFactory( )
				.newOdaDataSet( "DataSet" ); //$NON-NLS-1$
		designHandle.getDataSets( ).add( dataSetHandle );

		// Add one parameter to data set

		OdaDataSetParameter parameter = StructureFactory
				.createOdaDataSetParameter( );
		parameter.setName( "param1" ); //$NON-NLS-1$
		PropertyHandle parameterHandle = dataSetHandle
				.getPropertyHandle( DataSetHandle.PARAMETERS_PROP );
		parameterHandle.addItem( parameter );

		// Add one param binding to data set

		ParamBinding dataSetBinding = StructureFactory.createParamBinding( );
		dataSetBinding.setParamName( "param1" ); //$NON-NLS-1$
		PropertyHandle dataSetBindingHandle = dataSetHandle
				.getPropertyHandle( DataSetHandle.PARAM_BINDINGS_PROP );
		dataSetBindingHandle.addItem( dataSetBinding );

		// Add one table to design

		TableHandle tableHandle = designHandle.getElementFactory( )
				.newTableItem( "Table" ); //$NON-NLS-1$
		designHandle.getBody( ).add( tableHandle );

		// Let table use data set and create one parameter binding

		tableHandle.setDataSet( dataSetHandle );
		ParamBinding tableBinding = StructureFactory.createParamBinding( );
		tableBinding.setParamName( "param1" ); //$NON-NLS-1$
		PropertyHandle tableBindingHandle = tableHandle
				.getPropertyHandle( TableHandle.PARAM_BINDINGS_PROP );
		tableBindingHandle.addItem( tableBinding );

		// Add one image to design

		ImageHandle imageHandle = designHandle.getElementFactory( ).newImage(
				"Image" ); //$NON-NLS-1$
		designHandle.getBody( ).add( imageHandle );

		// Let image use data set and create one action

		imageHandle.setDataSet( dataSetHandle );
		Action action = StructureFactory.createAction( );
		imageHandle.setAction( action );

		// Add one parameter binding in action

		ActionHandle actionHandle = imageHandle.getActionHandle( );
		ParamBinding actionBinding = StructureFactory.createParamBinding( );
		actionBinding.setParamName( "param1" ); //$NON-NLS-1$
		actionHandle.addParamBinding( actionBinding );

		// Rename parameter name and parameter bindings are updated

		parameterHandle.getAt( 0 ).setProperty( DataSetParameter.NAME_MEMBER,
				"param2" ); //$NON-NLS-1$
		assertEquals( "param2", dataSetBinding.getParamName( ) ); //$NON-NLS-1$
		assertEquals( "param2", tableBinding.getParamName( ) ); //$NON-NLS-1$
		assertEquals( "param2", actionBinding.getParamName( ) ); //$NON-NLS-1$

		// Remove parameter name and parameter bindings are updated

		parameterHandle.removeItem( 0 );
		assertEquals( 0, dataSetBindingHandle.getListValue( ).size( ) );
		assertEquals( 0, tableBindingHandle.getListValue( ).size( ) );
		assertEquals( 0, actionHandle.getMember( Action.PARAM_BINDINGS_MEMBER )
				.getListValue( ).size( ) );
	}

	/**
	 * Returns the data set for testing.
	 * 
	 * @return the data set for testing.
	 * @throws Exception
	 *             if any exception.
	 */

	private OdaDataSetHandle getDataSet( ) throws Exception
	{
		openDesign( fileName );

		OdaDataSetHandle dataSet = (OdaDataSetHandle) designHandle
				.findDataSet( "firstDataSet" ); //$NON-NLS-1$
		assertNotNull( dataSet );

		return dataSet;
	}
}