/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.model.library;

import java.util.List;
import com.ibm.icu.util.ULocale;

import org.apache.commons.codec.binary.Base64;
import org.eclipse.birt.report.model.api.PropertyHandle;
import org.eclipse.birt.report.model.api.SlotHandle;
import org.eclipse.birt.report.model.api.ThemeHandle;
import org.eclipse.birt.report.model.api.elements.structures.ConfigVariable;
import org.eclipse.birt.report.model.api.elements.structures.CustomColor;
import org.eclipse.birt.report.model.api.elements.structures.EmbeddedImage;
import org.eclipse.birt.report.model.elements.Library;
import org.eclipse.birt.report.model.util.BaseTestCase;

/**
 * Tests the properties of library in reading and writing. This test case also
 * includes testing semantic check.
 */

public class LibraryParseTest extends BaseTestCase
{

	private String fileName = "LibraryParseTest.xml"; //$NON-NLS-1$
	private String outFileName = "LibraryParseTest_out.xml"; //$NON-NLS-1$
	private String goldenFileName = "LibraryParseTest_golden.xml"; //$NON-NLS-1$
	private String semanticCheckFileName = "LibraryParseTest_1.xml"; //$NON-NLS-1$

	/**
	 * Tests all properties and slots.
	 * 
	 * @throws Exception
	 *             if any exception
	 */

	public void testParser( ) throws Exception
	{
		openLibrary( fileName, ULocale.ENGLISH );

		assertEquals(
				"W.C. Fields", libraryHandle.getStringProperty( Library.AUTHOR_PROP ) ); //$NON-NLS-1$
		assertEquals(
				"http://company.com/reportHelp.html", libraryHandle.getStringProperty( Library.HELP_GUIDE_PROP ) ); //$NON-NLS-1$
		assertEquals(
				"Whiz-Bang Plus", libraryHandle.getStringProperty( Library.CREATED_BY_PROP ) ); //$NON-NLS-1$

		// title

		assertEquals(
				"TITLE_ID", libraryHandle.getStringProperty( Library.TITLE_ID_PROP ) ); //$NON-NLS-1$
		assertEquals(
				"Sample Report", libraryHandle.getStringProperty( Library.TITLE_PROP ) ); //$NON-NLS-1$

		// comments

		assertEquals(
				"First sample report.", libraryHandle.getStringProperty( Library.COMMENTS_PROP ) ); //$NON-NLS-1$

		// description

		assertEquals(
				"DESCRIP_ID", libraryHandle.getStringProperty( Library.DESCRIPTION_ID_PROP ) ); //$NON-NLS-1$
		assertEquals(
				"This is a first sample report.", libraryHandle.getStringProperty( Library.DESCRIPTION_PROP ) ); //$NON-NLS-1$

		// color-palette

		PropertyHandle colorPalette = libraryHandle
				.getPropertyHandle( Library.COLOR_PALETTE_PROP );
		List colors = colorPalette.getListValue( );
		assertEquals( 2, colors.size( ) );
		CustomColor color = (CustomColor) colors.get( 0 );
		assertEquals( "cus red", color.getName( ) ); //$NON-NLS-1$
		assertEquals( 111, color.getRGB( ) );
		assertEquals( "cus red key", color.getDisplayNameID( ) ); //$NON-NLS-1$
		assertEquals( "cus red display", color.getDisplayName( ) ); //$NON-NLS-1$
		color = (CustomColor) colors.get( 1 );
		assertEquals( "cus blue", color.getName( ) ); //$NON-NLS-1$
		assertEquals( 222, color.getRGB( ) );
		assertEquals( "cus blue key", color.getDisplayNameID( ) ); //$NON-NLS-1$

		// config-vars
		PropertyHandle configVarHandle = libraryHandle
				.getPropertyHandle( Library.CONFIG_VARS_PROP );
		List configVars = configVarHandle.getListValue( );
		assertEquals( 4, configVars.size( ) );
		ConfigVariable var = (ConfigVariable) configVars.get( 0 );
		assertEquals( "var1", var.getName( ) ); //$NON-NLS-1$
		assertEquals( "mumble.jpg", var.getValue( ) ); //$NON-NLS-1$
		var = (ConfigVariable) configVars.get( 1 );
		assertEquals( "var2", var.getName( ) ); //$NON-NLS-1$
		assertEquals( "abcdefg", var.getValue( ) ); //$NON-NLS-1$
		var = (ConfigVariable) configVars.get( 2 );
		assertEquals( "var3", var.getName( ) ); //$NON-NLS-1$
		assertEquals( "", var.getValue( ) ); //$NON-NLS-1$
		var = (ConfigVariable) configVars.get( 3 );
		assertEquals( "var4", var.getName( ) ); //$NON-NLS-1$
		assertEquals( null, var.getValue( ) );

		// images
		PropertyHandle imageHandle = libraryHandle
				.getPropertyHandle( Library.IMAGES_PROP );
		List images = imageHandle.getListValue( );
		assertEquals( 3, images.size( ) );
		EmbeddedImage image = (EmbeddedImage) images.get( 0 );
		assertEquals( "image1", image.getName( ) ); //$NON-NLS-1$
		assertEquals( "image/bmp", image.getType( libraryHandle.getModule( ) ) ); //$NON-NLS-1$		
		assertEquals( "imagetesAAA", //$NON-NLS-1$
				new String( Base64.encodeBase64( image.getData( libraryHandle
						.getModule( ) ) ) ).substring( 0, 11 ) );

		image = (EmbeddedImage) images.get( 1 );
		assertEquals( "image2", image.getName( ) ); //$NON-NLS-1$
		assertEquals( "image/gif", image.getType( libraryHandle.getModule( ) ) ); //$NON-NLS-1$
		assertEquals( "/9j/4AAQSkZJRgA", //$NON-NLS-1$
				new String( Base64.encodeBase64( image.getData( libraryHandle
						.getModule( ) ) ) ).substring( 0, 15 ) );

		image = (EmbeddedImage) images.get( 2 );
		assertEquals( "image3", image.getName( ) ); //$NON-NLS-1$
		assertEquals( "image/bmp", image.getType( libraryHandle.getModule( ) ) ); //$NON-NLS-1$
		assertEquals( "AAAA", //$NON-NLS-1$
				new String( Base64.encodeBase64( image.getData( libraryHandle
						.getModule( ) ) ) ) );

		SlotHandle themes = libraryHandle.getThemes( );
		assertEquals( 3, themes.getCount( ) );

		ThemeHandle theme = (ThemeHandle) themes.get( 0 );
		assertEquals( 1, theme.getStyles( ).getCount( ) );

		theme = (ThemeHandle) themes.get( 1 );
		assertEquals( 2, theme.getStyles( ).getCount( ) );
	}

	/**
	 * Tests writing the properties.
	 * 
	 * @throws Exception
	 *             if any error found.
	 */

	public void testWriter( ) throws Exception
	{
		openLibrary( fileName, ULocale.ENGLISH );

		libraryHandle.setProperty( Library.AUTHOR_PROP, "Report Author" ); //$NON-NLS-1$
		libraryHandle.setProperty( Library.HELP_GUIDE_PROP, "Help guide" ); //$NON-NLS-1$
		libraryHandle.setProperty( Library.CREATED_BY_PROP, "Report Creator" ); //$NON-NLS-1$
		libraryHandle.setProperty( Library.BASE_PROP, "c:\\base" ); //$NON-NLS-1$
		libraryHandle.setProperty( Library.UNITS_PROP, "cm" ); //$NON-NLS-1$
		libraryHandle.setProperty( Library.INCLUDE_RESOURCE_PROP,
				"new include resource" ); //$NON-NLS-1$

		libraryHandle.setProperty( Library.TITLE_ID_PROP, "New title id" ); //$NON-NLS-1$
		libraryHandle.setProperty( Library.TITLE_PROP, "New title" ); //$NON-NLS-1$
		libraryHandle.setProperty( Library.COMMENTS_PROP, "New comments" ); //$NON-NLS-1$
		libraryHandle.setProperty( Library.DESCRIPTION_ID_PROP,
				"New description id" ); //$NON-NLS-1$
		libraryHandle.setProperty( Library.DESCRIPTION_PROP, "New description" ); //$NON-NLS-1$

		saveLibraryAs( outFileName );
		assertTrue( compareTextFile( goldenFileName, outFileName ) );
	}

	/**
	 * Test config variable.
	 * 
	 * @throws Exception
	 *             if any error found.
	 */
	public void testConfigVars( ) throws Exception
	{
		openLibrary( fileName, ULocale.ENGLISH );

		ConfigVariable configVar = new ConfigVariable( );
		configVar.setName( "VarToAdd" ); //$NON-NLS-1$
		configVar.setValue( "ValueToAdd" ); //$NON-NLS-1$

		ConfigVariable newConfigVar = new ConfigVariable( );
		newConfigVar.setName( "VarToReplace" ); //$NON-NLS-1$
		newConfigVar.setValue( "ValueToReplace" ); //$NON-NLS-1$
	}

	/**
	 * Checks the semantic error of Library.
	 * 
	 * @throws Exception
	 *             if any exception
	 */

	public void testSemanticError( ) throws Exception
	{
		openLibrary( semanticCheckFileName );
		printSemanticErrors( );
		// assertEquals( 5, errors.size( ) );
		//
		// int i = 0;
		//
		// assertEquals( PropertyValueException.DESIGN_EXCEPTION_VALUE_REQUIRED,
		// ( (ErrorDetail) errors.get( i++ ) ).getErrorCode( ) );
		// assertEquals( PropertyValueException.DESIGN_EXCEPTION_VALUE_REQUIRED,
		// ( (ErrorDetail) errors.get( i++ ) ).getErrorCode( ) );
		// assertEquals( PropertyValueException.DESIGN_EXCEPTION_VALUE_EXISTS,
		// ( (ErrorDetail) errors.get( i++ ) ).getErrorCode( ) );
		// assertEquals( PropertyValueException.DESIGN_EXCEPTION_VALUE_REQUIRED,
		// ( (ErrorDetail) errors.get( i++ ) ).getErrorCode( ) );
		// assertEquals( SemanticError.DESIGN_EXCEPTION_MISSING_MASTER_PAGE,
		// ( (ErrorDetail) errors.get( i++ ) ).getErrorCode( ) );
	}

}